@@ -37,35 +37,40 @@ import vobject
 
 logger = logging.getLogger('etesync-dav')
 
-CONFIG_SECTION = "storage"
 
-
-# How often we should sync, in seconds
+# How often we should sync automatically, in seconds
 SYNC_INTERVAL = 15 * 60
 # Minimum time to wait between syncs
-SYNC_MINIMUM = 2 * 60
+SYNC_MINIMUM = 30
 
 
 class SyncThread(threading.Thread):
     def __init__(self, user, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self._force_sync = threading.Event()
+        self._done_syncing = threading.Event()
+        self._done_syncing.set()  # We are done before we start.
         self.user = user
         self.last_sync = None
 
     def request_sync(self):
         if self.last_sync and time.time() - self.last_sync >= SYNC_MINIMUM:
             self._force_sync.set()
+            self._done_syncing.clear()
 
     @property
     def forced_sync(self):
         return self._force_sync.is_set()
 
+    def wait_for_sync(self, timeout=None):
+        return self._done_syncing.wait(timeout)
+
     def run(self):
         while True:
             try:
                 with etesync_for_user(self.user) as (etesync, _):
                     self.last_sync = time.time()
+                    self._done_syncing.clear()
 
                     etesync.sync()
             except Exception as e:
@@ -73,6 +78,7 @@ class SyncThread(threading.Thread):
                 logger.exception(e)
             finally:
                 self._force_sync.clear()
+                self._done_syncing.set()
 
             self._force_sync.wait(SYNC_INTERVAL)
 
@@ -467,7 +473,7 @@ class Storage(BaseStorage):
 
     _collection_class = Collection
 
-    _lock = threading.RLock()
+    _sync_thread_lock = threading.RLock()
 
     def __init__(self, configuration):
         self.user = None
@@ -583,17 +589,21 @@ class Storage(BaseStorage):
             return
 
         with etesync_for_user(user) as (etesync, _):
-            with self._lock:
-                self.user = user
-                self.etesync = etesync
+            with self.__class__._sync_thread_lock:
+                if not hasattr(etesync, 'sync_thread'):
+                    etesync.sync_thread = SyncThread(user, daemon=True)
+                    etesync.sync_thread.start()
+                else:
+                    etesync.sync_thread.request_sync()
 
-                yield
+        # At most wait for 5 seconds before returning stale data
+        etesync.sync_thread.wait_for_sync(5)
 
-                self.etesync = None
-                self.user = None
+        with etesync_for_user(user) as (etesync, _):
+            self.user = user
+            self.etesync = etesync
 
-            if not hasattr(etesync, 'sync_thread'):
-                etesync.sync_thread = SyncThread(user, daemon=True)
-                etesync.sync_thread.start()
-            else:
-                etesync.sync_thread.request_sync()
+            yield
+
+            self.etesync = None
+            self.user = None
